pub struct Ghost<T: ?Sized>(/* private fields */);Expand description
A type that can be used in ghost! context.
This type may be used to make more complicated proofs possible. In particular, some proof may need a notion of non-duplicable token to carry around.
Conceptually, a Ghost<T> is an object of type T that resides in a special “ghost”
heap. This heap is inaccessible from normal code, and Ghost values cannot be used
to influence the behavior of normal code.
This box can be accessed in a ghost! block:
ⓘ
let b: Ghost<i32> = Ghost::new(1);
ghost! {
let value: i32 = b.into_inner();
// use value here
}
let value: i32 = b.into_inner(); // compile error !Implementations
Sourceimpl<T: ?Sized> Ghost<T>
impl<T: ?Sized> Ghost<T>
Sourcepub fn borrow_mut(&mut self) -> Ghost<&mut T>
pub fn borrow_mut(&mut self) -> Ghost<&mut T>
Transforms a &mut Ghost<T> into a Ghost<&mut T>.