Trait NthBitLogic
Source pub trait NthBitLogic {
// Required method
fn nth_bit(self, n: Int) -> bool;
}
Trait for the nth bit of a bitvector in logic code.
The nth bit of a bitvector.
In bitwise_proof mode, this is translated to nth in Why3.
Otherwise this is an abstract function.
logic ⚠