SumLocalUpdateR

Struct SumLocalUpdateR 

pub struct SumLocalUpdateR<U>(pub U);
Apply an update to the Right variant of an authority/fragment pair of Sums.

This requires either the authority or the fragment to be in the Right state (the other will be implied as they must always be in the same state).

Tuple Fields

0: U