SumUpdateL

Struct SumUpdateL 

pub struct SumUpdateL<U>(pub U);
Apply an update to the left side of a Sum.

This requires the resource to be in the Left state.

Example

use creusot_std::{
    ghost::resource::Resource,
    logic::ra::{
        excl::{Excl, ExclUpdate},
        sum::{Sum, SumUpdateL},
    },
    prelude::*,
};

let mut res: Ghost<Resource<Sum<Excl<Int>, Excl<()>>>> =
    Resource::alloc(snapshot!(Sum::Left(Excl(1))));
ghost! { res.update(SumUpdateL(ExclUpdate(snapshot!(2)))) };
proof_assert!(res@ == Sum::Left(Excl(2)));

Tuple Fields

0: U