Int

Struct Int 

pub struct Int;
An unbounded, mathematical integer.

This type cannot be only be constructed in logical or ghost code.

Integers in pearlite

Note that in pearlite, all integer literals are of type Int:

let x = 1i32;
//             ↓ need to use the view operator to convert `i32` to `Int`
proof_assert!(x@ == 1);

You can use the usual operators on integers: +, -, *, / and %.

Note that those operators are not available in ghost code.

Implementations

Source

impl Int

pub fn pow(self, p: Int) -> Int

Compute self^p.

Example
proof_assert!(3.pow(4) == 729);

logic

pub fn pow2(self) -> Int

Compute 2^p.

Example
proof_assert!(pow2(4) == 16);

logic

pub fn max(self, x: Int) -> Int

Compute the maximum of self and x.

Example
proof_assert!(10.max(2) == 10);

logic

pub fn min(self, x: Int) -> Int

Compute the minimum of self and x.

Example
proof_assert!(10.max(2) == 2);

logic

pub fn div_euclid(self, d: Int) -> Int

Compute the euclidean division of self by d.

Example
proof_assert!(10.div_euclid(3) == 3);

logic

pub fn rem_euclid(self, d: Int) -> Int

Compute the remainder of the euclidean division of self by d.

Example
 proof_assert!(10.rem_euclid(3) == 1);

logic

pub fn abs_diff(self, other: Int) -> Int

Compute the absolute difference of self and x.

Example
proof_assert!(10.abs_diff(3) == 7);
proof_assert!(3.abs_diff(10) == 7);
proof_assert!((-5).abs_diff(5) == 10);

logic(open)

if self < other { other - self } else { self - other }

Source

impl Int

pub fn new(value: i128) -> Ghost<Self>

Create a new Int value

The result is wrapped in a Ghost, so that it can only be access inside a ghost! block.

You should not have to use this method directly: instead, use the int suffix inside of a ghost block:

let x: Ghost<Int> = ghost!(1int);
ghost! {
    let y: Int = 2int;
};

terminates

ghost

ensures

*result == value@

pub fn incr_ghost(&mut self)

terminates

ghost

ensures

^self == *self + 1

pub fn decr_ghost(&mut self)

terminates

ghost

ensures

^self == *self - 1