pub trait Resolve { }Expand description
The trait Resolve makes it possible to expose to clients a custom resolve assertion for
opaque types.
For example, if a library defines a notion of finite mapping, it does not exposes the internal
representation of the finite mapping data structure. Hence, the resolve predicate above wil be
completely opaque for clients. This library should implement the Resolve trait in order to
provide to the client a definition it can use. E.g., Resolve::resolve states that any element
of the mapping is resolved.
Implementations on Foreign Types
impl<'a, T> Resolve for IterMut<'a, T>
impl<'a, T: ?Sized> Resolve for &'a T
impl<'a, T: ?Sized> Resolve for &'a mut T
impl<I> Resolve for Cloned<I>
impl<I> Resolve for Copied<I>
impl<I> Resolve for Enumerate<I>
impl<I> Resolve for Skip<I>
impl<I> Resolve for Take<I>
impl<I, F> Resolve for Map<I, F>
impl<T> Resolve for [T]
impl<T> Resolve for VecDeque<T>
impl<T> Resolve for IntoIter<T>
impl<T> Resolve for Vec<T>
impl<T, const N: usize> Resolve for [T; N]
impl<T: ?Sized> Resolve for Box<T>
Available on non-crate feature
nightly only.