pub trait FnExt<Args>: FnMutExt<Args> { }Expand description
FnExt is an extension trait for the Fn trait, used for
adding a specification to closures. It should not be used directly.
pub trait FnExt<Args>: FnMutExt<Args> { }FnExt is an extension trait for the Fn trait, used for
adding a specification to closures. It should not be used directly.