pub fn spawn<F, T>(f: F) -> JoinHandle<T>Expand description
Creusot wrapper around std::thread::spawn.
The only difference is that the closure gives access to a fresh token object
pub fn spawn<F, T>(f: F) -> JoinHandle<T>Creusot wrapper around std::thread::spawn.
The only difference is that the closure gives access to a fresh token object