pub struct Id(/* private fields */);Expand description
A unique id, usable in logic and ghost code
The only properties of this type are:
- there is an infinite number of ids
- You can always generate a fresh id in logic code
Usage
Ids are used in PermCell and resource algebras.