Snapshot

Struct Snapshot 

pub struct Snapshot<T: ?Sized>(/* private fields */);
A copyable snapshot, usable in pearlite.

The Snapshot type contains the logical value of some data, in a purely immutable way. It is zero sized.

Creating a snapshot does not move the ownership of a value.

Pearlite syntax

In executable code, you may create a snapshot with the snapshot! macro. Inside this macro, you may write pearlite code; this code will not run during the normal execution of the program.

Example

let x: Snapshot<Int> = snapshot!(1);
let m: Snapshot<Mapping<Int, Int>> = snapshot!(|x| x + 1);

Implementations

Source

impl<T: ?Sized> Snapshot<T>

pub fn new(value: T) -> Snapshot<T>

Create a new snapshot in logic code.

logic

Source

impl<T> Snapshot<T>

pub fn inner(self) -> T

Get the value of the snapshot.

When possible, you should instead use the dereference operator.

Example
let x = snapshot!(1);
proof_assert!(x.inner() == 1);
proof_assert!(*x == 1); // prefer this

logic

pub fn into_ghost(self) -> Ghost<T>
where T: Plain,

Extract a plain value from a snapshot in ghost code.

ensures

*result == *self

terminates

ghost