pub struct Snapshot<T: ?Sized>(/* private fields */);Expand description
A copyable snapshot, usable in pearlite.
The Snapshot type contains the logical value of some data, in a purely immutable way.
It is zero sized.
Creating a snapshot does not move the ownership of a value.
Pearlite syntax
In executable code, you may create a snapshot with the snapshot! macro. Inside
this macro, you may write pearlite code; this code will not run during the normal
execution of the program.
Example
let x: Snapshot<Int> = snapshot!(1);
let m: Snapshot<Mapping<Int, Int>> = snapshot!(|x| x + 1);Implementations